=== modified file 'NEWS'
--- NEWS	2006-08-25 14:36:23 +0000
+++ NEWS	2006-08-25 19:51:41 +0000
@@ -23,6 +23,10 @@
 
   BUG FIXES:
 
+    * Change LockDir so that if the lock directory doesn't exist when
+      lock_write() is called, an attempt will be made to create it.
+      (John Arbash Meinel, #56974)
+
     * ``bzr uncommit`` preserves pending merges. (John Arbash Meinel, #57660)
 
     * Active FTP transport now works as intended. (ghozzy, #56472)

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2006-07-28 16:05:23 +0000
+++ bzrlib/lockdir.py	2006-08-24 20:04:13 +0000
@@ -189,7 +189,17 @@
             raise UnlockableTransport(self.transport)
         try:
             tmpname = '%s/pending.%s.tmp' % (self.path, rand_chars(20))
-            self.transport.mkdir(tmpname)
+            try:
+                self.transport.mkdir(tmpname)
+            except NoSuchFile:
+                # This may raise a FileExists exception
+                # which is okay, it will be caught later and determined
+                # to be a LockContention.
+                self.create(mode=self._dir_modebits)
+                
+                # After creating the lock directory, try again
+                self.transport.mkdir(tmpname)
+
             sio = StringIO()
             self._prepare_info(sio)
             sio.seek(0)
@@ -198,6 +208,7 @@
             # into place, because that's going to happen to the whole
             # directory
             self.transport.append(tmpname + self.__INFO_NAME, sio)
+
             self.transport.rename(tmpname, self._held_dir)
             self._lock_held = True
             self.confirm()

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_lockdir.py	2006-08-24 20:04:13 +0000
@@ -352,3 +352,27 @@
             self.assertRaises(LockBroken, ld1.unlock)
         finally:
             bzrlib.ui.ui_factory = orig_factory
+
+    def test_create_missing_base_directory(self):
+        """If LockDir.path doesn't exist, it can be created
+
+        Some people manually remove the entire lock/ directory trying
+        to unlock a stuck repository/branch/etc. Rather than failing
+        after that, just create the lock directory when needed.
+        """
+        t = self.get_transport()
+        lf1 = LockDir(t, 'test_lock')
+
+        lf1.create()
+        self.failUnless(t.has('test_lock'))
+
+        t.rmdir('test_lock')
+        self.failIf(t.has('test_lock'))
+
+        # This will create 'test_lock' if it needs to
+        lf1.lock_write()
+        self.failUnless(t.has('test_lock'))
+        self.failUnless(t.has('test_lock/held/info'))
+
+        lf1.unlock()
+        self.failIf(t.has('test_lock/held/info'))

